@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-
+#define USE_RINTERNALS
 #include <Rinternals.h>
 #include <R_ext/Mathlib.h>
 
@@ -186,7 +186,7 @@ get_string_type (SEXP call, SEXP vec)
 
   for (vecpos = 0; vecpos < LENGTH (vec); vecpos++)
     {
-      SEXP stritem = STRING (vec)[vecpos];
+      SEXP stritem = STRING_ELT (vec, vecpos);
 
       if (LENGTH (stritem) > maxstrlen)
 	maxstrlen = LENGTH (stritem);
@@ -219,7 +219,7 @@ vector_io (SEXP call, int writeflag, hid_t dataset, hid_t space, SEXP obj)
 
   case STRSXP:
       memtid = make_sexp_ref_type (call);
-      buf = STRING (obj);
+      buf = STRING_PTR (obj);
       break;
   case REALSXP:
       memtid = H5T_NATIVE_DOUBLE;
@@ -307,7 +307,7 @@ hdf5_save_attributes (SEXP call, hid_t loc_id, SEXP val)
 	{
 	  memtid = make_sexp_ref_type (call);
 	  tid = get_string_type (call, attr);
-	  buf = STRING (attr);
+	  buf = STRING_PTR (attr);
 	}
       else if (type == LGLSXP)
 	{
@@ -486,7 +486,7 @@ create_rownames_dataset_attribute (SEXP call, hid_t dataset, SEXP rownames)
 			     stringtid, rndataspace, H5P_DEFAULT)) < 0)
     errorcall (call, "unable to create row names dataset");
 
-  if (H5Awrite (rnattrib, rtid, STRING (rownames)) < 0)
+  if (H5Awrite (rnattrib, rtid, STRING_PTR (rownames)) < 0)
     errorcall (call, "unable to write row names dataset");
 
   if (H5Aclose (rnattrib) < 0)
@@ -522,7 +522,7 @@ hdf5_save_object (SEXP call, hid_t fid, const char *symname, SEXP val)
 
       for (pos = 0; pos < colcount; pos++)
 	{
-	  SEXPTYPE type = TYPEOF (VECTOR (val)[pos]);
+	  SEXPTYPE type = TYPEOF (VECTOR_ELT (val, pos));
 
 	  switch (type)
 	    {
@@ -533,7 +533,7 @@ hdf5_save_object (SEXP call, hid_t fid, const char *symname, SEXP val)
 	      hdftypes[pos] = H5T_NATIVE_INT;
 	      break;
 	    case STRSXP:
-	      hdftypes[pos] = get_string_type (call, VECTOR (val)[pos]);
+	      hdftypes[pos] = get_string_type (call, VECTOR_ELT (val, pos));
 	      break;
 	    case LGLSXP:
 	      hdftypes[pos] = hdfbooltype;
@@ -556,7 +556,7 @@ hdf5_save_object (SEXP call, hid_t fid, const char *symname, SEXP val)
 	hid_t ctid;
 	hid_t dataset;
 	hid_t dataspace;
-	unsigned rowcount = length (VECTOR (val)[0]);
+	unsigned rowcount = length (VECTOR_ELT (val, 0));
 	{
 	  hsize_t dims[1];
 
@@ -573,7 +573,7 @@ hdf5_save_object (SEXP call, hid_t fid, const char *symname, SEXP val)
 
 	  for (pos = 0; pos < colcount; pos++)
 	    if (H5Tinsert (ctid,
-			   CHAR (STRING (colnames)[pos]),
+			   CHAR (STRING_ELT (colnames, pos)),
 			   offsets[pos],
 			   hdftypes[pos]) < 0)
 	      errorcall (call, "unable to insert type into compound type");
@@ -595,7 +595,7 @@ hdf5_save_object (SEXP call, hid_t fid, const char *symname, SEXP val)
 	  for (ri = 0; ri < rowcount; ri++)
 	    for (pos = 0; pos < colcount; pos++)
 	      {
-		SEXP item = VECTOR (val)[pos];
+		SEXP item = VECTOR_ELT (val, pos);
 		SEXPTYPE type = TYPEOF (item);
 		void *ptr = &buf[ri][offsets[pos]];
 
@@ -609,7 +609,7 @@ hdf5_save_object (SEXP call, hid_t fid, const char *symname, SEXP val)
 		    break;
 		  case STRSXP:
 		    {
-		      SEXP stritem = STRING (item)[ri];
+		      SEXP stritem = STRING_ELT (item, ri);
 
 		      memset (ptr, 0, H5Tget_size (hdftypes[pos]));
 		      strcpy ((char *)ptr, CHAR (stritem));
@@ -695,12 +695,12 @@ hdf5_save_object (SEXP call, hid_t fid, const char *symname, SEXP val)
 	      {
 		for (pos = 0; pos < len; pos++)
 		  {
-		    SEXP s = VECTOR (val)[pos];
+		    SEXP s = VECTOR_ELT (val, pos);
 		    SEXP names = getAttrib (val, R_NamesSymbol);
 
 		    if (!isNull (names))
 		      hdf5_save_object (call, gid,
-				       CHAR (STRING (names) [pos]),
+				       CHAR (STRING_ELT (names, pos)),
 				       s);
 		    else
 		      {
@@ -776,7 +776,7 @@ do_hdf5save (SEXP args)
   if (TYPEOF (CAR (args)) != STRSXP)
     errorcall (call, "first argument must be a pathname");
 
-  path = CHAR (STRING (CAR (args))[0]);
+  path = CHAR (STRING_ELT (CAR (args), 0));
 
   H5dont_atexit ();
 
@@ -802,7 +802,7 @@ do_hdf5save (SEXP args)
       if (TYPEOF (CAR (names)) != STRSXP)
         errorcall (call, "expecting a symbol name");
       {
-        char *name = CHAR (STRING (CAR (names))[0]);
+        char *name = CHAR (STRING_ELT (CAR (names), 0));
         
         PROTECT (sym = install (name));
         PROTECT (s = findVar (sym, env));
@@ -912,7 +912,7 @@ load_rownames_dataset_attribute (SEXP call, hid_t dataset, SEXP vec)
         for (i = 0; i < rowcount; i++)
           {
             sprintf (buf, "%u", i);
-            STRING (rownames)[i] = mkChar (buf);
+            SET_STRING_ELT (rownames, i, mkChar (buf));
           }
       }      
       setAttrib (vec, R_RowNamesSymbol, rownames);
@@ -943,13 +943,13 @@ load_rownames_dataset_attribute (SEXP call, hid_t dataset, SEXP vec)
     hid_t rtid = make_sexp_ref_type (call);
 
     for (ri = 0; ri < rowcount; ri++)
-      strptrs[ri] = STRING (rownames)[ri];
+      strptrs[ri] = STRING_ELT (rownames, ri);
 
     if (H5Aread (rnattrib, rtid, strptrs) < 0)
       errorcall (call, "can't read rownames");
 
     for (ri = 0; ri < rowcount; ri++)
-      STRING (rownames)[ri] = strptrs [ri];
+      SET_STRING_ELT (rownames, ri, strptrs [ri]);
   }
   setAttrib (vec, R_RowNamesSymbol, rownames);
   UNPROTECT (1);
@@ -1039,7 +1039,7 @@ hdf5_process_attribute (hid_t loc_id, const char *attrName, void *data)
 	      buf = REAL (vec);
 	      break;
 	    case H5T_STRING:
-	      buf = STRING (vec);
+	      buf = STRING_PTR (vec);
 	      break;
 	    default:
 	      abort ();
@@ -1187,7 +1187,7 @@ hdf5_process_object (hid_t id, const char *name, void *client_data)
 		H5T_class_t class = H5Tget_class (ctid);
 		size_t csize = H5Tget_size (ctid);
 		size_t coffset = H5Tget_member_offset (tid, ci);
-		SEXPREC **rowptr = &VECTOR (vec)[ci];
+		SEXPREC **rowptr = &VECTOR_ELT (vec, ci);
 		unsigned char itembuf[size]; /* for overrun */
 
 #define VECLOOP(vectype, vecref, dtid) \
@@ -1206,7 +1206,7 @@ hdf5_process_object (hid_t id, const char *name, void *client_data)
 		  char *colname = H5Tget_member_name (tid, ci);
 
 		  if (colname)
-		    STRING (names)[ci] = mkChar (colname);
+		    SET_STRING_ELT (names, ci, mkChar (colname));
 		}
 		switch (class) {
 		  case H5T_INTEGER:
@@ -1223,7 +1223,7 @@ hdf5_process_object (hid_t id, const char *name, void *client_data)
 		    break;
 		  case H5T_STRING:
 		    *rowptr = allocVector (STRSXP, rowcount);
-		    VECLOOP (STRSXP, STRING, rtid);
+		    VECLOOP (STRSXP, STRING_PTR, rtid);
 		    break;
 		  default:
 		    errorcall (iinfo->call, "can't handle hdf class %d",
@@ -1303,7 +1303,7 @@ SEXP do_hdf5load (SEXP args)
   if (TYPEOF (CADR (args)) != LGLSXP)
     errorcall (call, "second argument must be a logical vector");
 
-  path = CHAR(STRING (CAR(args))[0]);
+  path = CHAR(STRING_ELT (CAR(args), 0));
   restore_syms = INTEGER (CADR(args))[0];
 
   H5dont_atexit ();
